"""Simple struct operations to pack and unpack numbers to strings."""

# The copyright in this software is being made available under the BSD License,
# included below. This software may be subject to other third party and contributor
# rights, including patent rights, and no such rights are granted under this license.
#
# Copyright (c) 2016, Dash Industry Forum.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#  * Redistributions of source code must retain the above copyright notice, this
#  list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation and/or
#  other materials provided with the distribution.
#  * Neither the name of Dash Industry Forum nor the names of its
#  contributors may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND ANY
#  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
#  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from struct import pack, unpack

def str_to_uint16(string2):
    "2-character string to unsigned int16."
    return unpack(">H", string2)[0]

def str_to_uint32(string4):
    "4-character string to unsigned int32."
    return unpack(">I", string4)[0]

def str_to_sint32(string4):
    "4-character string to signed int32."
    return unpack(">i", string4)[0]

def str_to_uint64(string8):
    "8-character string to unsigned int64."
    return unpack(">Q", string8)[0]

def uint16_to_str(uint16):
    "Unsigned int16 to string."
    return pack(">H", uint16)

def uint32_to_str(uint32):
    "Unsigned int32 to string."
    return pack(">I", uint32)

def sint32_to_str(sint32):
    "Signed int32 to string."
    return pack(">i", sint32)

def uint64_to_str(uint64):
    "Unsigned int64 to string."
    return pack(">Q", uint64)
